import json
import git
import requests
from flask import Response
from xml.etree import ElementTree as ET
import xmlschema

ns = {"w3":"http://www.w3.org/2001/XMLSchema"}

def get_config_dict():
    with open('./config.json') as json_config_file:
        config = json.load(json_config_file)

    return config

def git_cmd(config):
    return git.cmd.Git(config['git']['dir'])

def get_xsd_url(config, file_name):
    return "{uri}/{file_name}".format(uri=config['xsd']['url'], file_name=file_name)

def get_xsd_path_file(config, file_name):
    return "{dir}/{file_name}".format(dir=config['xsd']['dir'], file_name=file_name)

def get_xsd(config, from_server=True):
    file_name = config['xsd']['file']
    if from_server:
        url = get_xsd_url(config, file_name)
        xsd_str = requests.get(url).text
    else:
        path_file = get_xsd_path_file(config, file_name)
        xsd_str = open(path_file).read()

    return xsd_str

def save_xsd(config, xsd_str, file_name):
    path_file = get_xsd_path_file(config, file_name)
    file = open(path_file, "w")
    file.write(xsd_str)  
    
    return return_ok() 

def parse(xml_str):
    try:
        tree = ET.fromstring(xml_str)
    except ET.ParseError:
        tree = None
    return tree

def get_xsd_version(xsd_str):
    tree = parse(xsd_str)
    for annotation in tree.findall("w3:annotation", ns):
        version_text = annotation.find("w3:documentation", ns).text

    return version_text

def validate(xml_str, xsd_str):
    tree = parse(xml_str)
    schema = xmlschema.XMLSchema(xsd_str)
    try:
        schema.validate(tree) 
        return return_ok()
    except xmlschema.XMLSchemaValidationError as error:
        return return_error(error)
   
def xml_response(xml_str):
    return Response(xml_str, content_type='text/xml; charset=utf-8')

def return_ok():
    return '<ok/>'

def return_error(error):
    return '<error>{error}</error>'.format(error=error)

def return_version(version):
    return '<version>{version}</version>'.format(version=version)